# Section Box  Copyright (C) 2023  Ruben Messerschmidt

# Section Box is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Section Box is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.


bl_info = {
    "name": "Instant Clean",
    "description": "Clean Mesh in one Click",
    "blender": (2, 93, 0),
    "version": (2, 1, 2),
    "category": "Mesh",
    "author": "Ruben Messerschmidt",
    "location": "View3D > Sidebar > Instant Clean",
    "doc_url": "https://instant-clean.readthedocs.io/en/latest/",
    "version_url": "https://raw.githubusercontent.com/rubenmesserschmidt/Instant-Clean_docs/main/version",
}

addon_id = __package__

import os
from pathlib import Path

import bpy

from . import auto_load
from . import icons
from . import props
from . import update
from . import ui
from .utils import presets


MODULES = [
    icons,
    auto_load,
    props,
    ui,
]


auto_load.init()

def register():
    # register modules
    for module in MODULES:
        module.register()

    # presets
    presets_path = os.path.join(
        bpy.utils.user_resource("SCRIPTS", path="presets", create=True),
        "Instant Clean",
    )
    if not os.path.isdir(presets_path):
        presets.install_presets()

    for preset in Path(presets_path).glob("*.py"):
        with open(preset, "r") as file:
            filedata = file.read()

        if filedata.find("bc_states") == -1:
            continue

        filedata = filedata.replace("bc_states", "instantclean_categories")
        filedata = filedata.replace("bc_repair", "instantclean_repair")
        filedata = filedata.replace("bc_normals", "instantclean_normals")
        filedata = filedata.replace("bc_limdiss", "instantclean_dissolve")
        filedata = filedata.replace("bc_topology", "instantclean_topology")
        filedata = filedata.replace("bc_objectdata", "instantclean_objectdata")

        filedata = filedata.replace(
            "instantclean_categories.limdiss", "instantclean_categories.dissolve"
        )
        filedata = filedata.replace(
            "instantclean_objectdata.unused_material_slots",
            "instantclean_objectdata.material_slots",
        )

        if preset.name == "Ultimate.py":
            features = """
instantclean_manifold = bpy.context.scene.instantclean_manifold
instantclean_categories.manifold = True
instantclean_manifold.non_manifold_faces = True
instantclean_manifold.non_manifold_verts = True
instantclean_manifold.wire_geo = True
instantclean_manifold.fill_holes = True
instantclean_manifold.fill_holes_max_sides = 4
instantclean_manifold.flatten = True
instantclean_manifold.flatten_iterations = 3
instantclean_manifold.flatten_factor = 1.0
instantclean_manifold.undistort = True
instantclean_manifold.undistort_ang = 0.0872659981250763
instantclean_topology.quad_max_face_ang = 0.6981229782104492
instantclean_topology.quad_max_shape_ang = 0.6981229782104492
instantclean_objectdata.double_materials = True
instantclean_objectdata.double_materials_remove_data = False
instantclean_objectdata.material_slots_clear = 'UNUSED'
"""

        elif preset.name == "Basic.py":
            features = """
instantclean_manifold = bpy.context.scene.instantclean_manifold
instantclean_categories.manifold = True
instantclean_manifold.non_manifold_faces = True
instantclean_manifold.non_manifold_verts = False
instantclean_manifold.wire_geo = True
instantclean_manifold.fill_holes = False
instantclean_manifold.fill_holes_max_sides = 4
instantclean_manifold.flatten = False
instantclean_manifold.flatten_iterations = 3
instantclean_manifold.flatten_factor = 1.0
instantclean_manifold.undistort = False
instantclean_manifold.undistort_ang = 0.0872659981250763
instantclean_topology.quad_max_face_ang = 0.6981229782104492
instantclean_topology.quad_max_shape_ang = 0.6981229782104492
instantclean_objectdata.double_materials = True
instantclean_objectdata.double_materials_remove_data = False
instantclean_objectdata.material_slots_clear = 'UNUSED'
"""

        elif preset.name == "Light.py":
            features = """
instantclean_manifold = bpy.context.scene.instantclean_manifold
instantclean_categories.manifold = False
instantclean_manifold.non_manifold_faces = False
instantclean_manifold.non_manifold_verts = False
instantclean_manifold.wire_geo = False
instantclean_manifold.fill_holes = False
instantclean_manifold.fill_holes_max_sides = 4
instantclean_manifold.flatten = False
instantclean_manifold.flatten_iterations = 3
instantclean_manifold.flatten_factor = 1.0
instantclean_manifold.undistort = False
instantclean_manifold.undistort_ang = 0.0872659981250763
instantclean_topology.quad_max_face_ang = 0.6981229782104492
instantclean_topology.quad_max_shape_ang = 0.6981229782104492
instantclean_objectdata.double_materials = False
instantclean_objectdata.double_materials_remove_data = False
instantclean_objectdata.material_slots_clear = 'UNUSED'
"""

        filedata = f"{filedata}{features}"

        with open(preset, "w") as file:
            file.write(filedata)
            file.close()

def unregister():
    for module in reversed(MODULES):
        module.unregister()
