import bpy

from .ops.popup_base import PopupBase
from .utils.cache import Preferences
from .utils.preferences import get_prefs
from .utils.system import internet_connection
from .utils.ui import draw_paragraph, close_dialog
from .utils.version import (
    get_this_version_name,
    get_latest_version_name,
)
from . import addon_id, bl_info


show_popup = True


class UpdateCheck(PopupBase, bpy.types.Operator):
    bl_idname = f"{addon_id}.update_check"
    bl_description = "Check for updates (requires internet connection)"

    manual: bpy.props.BoolProperty(
        options={"HIDDEN"}
    )  # called manually from addon preferences
    version: bpy.props.StringProperty(options={"HIDDEN"})

    icon = "FILE_REFRESH"

    @classmethod
    def description(cls, context, properties):
        action = properties.action
        if action == "update":
            return f"Update {bl_info['name']} to the latest version now"
        elif action == "ignore":
            return "Ignore this update and don't notify again for this version"
        elif action == "remind":
            return "Remind me again on the next startup"

        return cls.bl_description

    @staticmethod
    def invoke_popup():
        prefs = get_prefs()
        if not prefs.update_check:
            return

        getattr(getattr(bpy.ops, addon_id), UpdateCheck.bl_idname.split(".")[-1])(
            "INVOKE_DEFAULT", manual=False
        )

    def execute(self, context):
        if self.action == "update":
            bpy.ops.wm.url_open(
                url="https://instant-clean.readthedocs.io/en/latest/update.html"
            )
        elif self.action == "ignore":
            self.report({"INFO"}, f"Ignoring update {self.version}")
            Preferences.set("ignore_version", self.version)
        elif self.action == "remind":
            self.report({"INFO"}, "You'll get notified again on the next startup.")

        return {"FINISHED"}

    def invoke(self, context, event):
        global show_popup

        self.prefs = get_prefs()

        if self.action != "init":
            show_popup = False
            self.closed = True
            close_dialog(context, event)
            return self.execute(context)

        if not internet_connection():
            self.report(
                {"ERROR"},
                "Could not connect to server, please check your internet connection!",
            )

        # TODO: Use version names and parsed versions to compare
        this_version = get_this_version_name()
        latest_version = get_latest_version_name()

        if not this_version or not latest_version or latest_version <= this_version:
            if self.manual:
                self.report({"INFO"}, f"{bl_info['name']} is up to date!")

            return {"CANCELLED"}

        if not self.manual:
            if not show_popup:
                return {"CANCELLED"}

            if Preferences.get("ignore_version") == latest_version:
                return {"CANCELLED"}

            context.window.cursor_warp(
                int(context.window.width / 2), int(context.window.height / 2)
            )

        self.version = ".".join(str(i) for i in latest_version)
        return super().invoke(context, event)

    def draw(self, context):
        super().draw(context)
        layout = self.layout
        text = (
            f"A new version of {bl_info['name']} is available!\n"
            "Update now and explore whats new in"
            f"\n{bl_info['name']} {self.version}"
        )
        draw_paragraph(layout, text, alignment="CENTER")

        op_col = layout.column(align=True)

        row = op_col.row(align=True)
        row.active_default = True
        op = row.operator(self.bl_idname, text="Update Instructions")
        op.action = "update"
        op.version = self.version

        row = op_col.row()
        row = op_col.row(align=True)
        op = row.operator(self.bl_idname, text="Remind Me Later")
        op.action = "remind"

        op = row.operator(self.bl_idname, text="Ignore This Update")
        op.action = "ignore"
        op.version = self.version
