import bpy

from .text import iter_lines, line_to_rows, draw_text
from ..icons import icon_id
from .. import addon_id


def sub_layout(
    layout: bpy.types.UILayout, spacing: tuple = (0, 0, 0), align: bool = False
):
    """Create a sub layout with separators

    :param layout: Layout to create the sub layout in
    :type layout: bpy.types.UILayout
    :param spacing: Spacing of the layout in the order of top, left, right, defaults to (0, 0, 0)
    :type spacing: tuple, optional
    :param align: Align layout, defaults to False
    :type align: bool, optional
    :return: Column of the sub layout
    :rtype: bpy.types.UILayout
    """
    col = layout.column()
    col.separator(factor=spacing[0])
    row = col.row(align=align)
    row.separator(factor=spacing[1])
    col = row.column()
    row.separator(factor=spacing[2])
    return col


levels_icon = [
    ("RIGHTARROW", "DOWNARROW_HLT"),
    ("DISCLOSURE_TRI_RIGHT", "DISCLOSURE_TRI_DOWN"),
]


def expand_layout(
    layout: bpy.types.UILayout,
    name: str,
    level: int,
    data,
    expand_prop: str,
    enable: bool = True,
) -> bpy.types.UILayout:
    """Create an expand layout

    :param layout: Layout to create the expand layout in
    :type layout: bpy.types.UILayout
    :param name: Name of the layout
    :type name: str
    :param level: Level of the layout: 0, 1 (different icons)
    :type level: int
    :param data: source data of the property
    :type data: AnyType
    :param expand_prop: expand property
    :type expand_prop: str
    :param enable: enable layout, defaults to True
    :type enable: bool, optional
    """
    row = layout.row(align=True)
    row.enabled = enable
    icon = (
        levels_icon[level][1] if getattr(data, expand_prop) else levels_icon[level][0]
    )
    row.prop(data, expand_prop, text="", icon=icon, emboss=False)
    row.label(text=name)
    return row


def split_layout(
    layout: bpy.types.UILayout,
    split_factor: float = 0.5,
    spacing: float = 0,
    align_left: bool = False,
    align_right: bool = False,
):
    split = layout.split(factor=split_factor)
    if spacing == 0:
        lcol = split.column(align=align_left)
    else:
        row = split.row()
        row.separator(factor=spacing)
        lcol = row.column(align=align_left)
    rcol = split.column(align=align_right)
    return lcol, rcol


def get_window_title():
    import pygetwindow as pw

    return pw.getActiveWindowTitle()


def get_icons():
    return (
        bpy.types.UILayout.bl_rna.functions["prop"]
        .parameters["icon"]
        .enum_items.values()
    )


def get_icon_value(icon):
    icons = get_icons()
    icon_item = [i for i in icons if i.identifier == icon]
    icon_value = 0
    if not icon_item:
        icon = icon_id(icon)
        if icon is not None:
            icon_value = icon
    else:
        icon_value = icon_item[0].value

    return icon_value


def get_panel_width(area):
    for r in area.regions:
        if r.type == "UI":
            return r.width


def get_area_name(area):
    type_name = {
        "VIEW_3D": "3D Viewport",
        "IMAGE_EDITOR": "Image Editor",
        "UV": "UV Editor",
        "ShaderNodeTree": "Shader Editor",
        "GeometryNodeTree": "Geometry Node Editor",
        "CompositorNodeTree": "Compositor Node Editor",
        "TextureNodeTree": "Texture Node Editor",
        "FILES": "File Browser",
        "SEQUENCE_EDITOR": "Video Sequence Editor",
        "TIMELINE": "Timeline",
        "GRAPH_EDITOR": "Graph Editor",
        "FCURVES": "Graph Editor",
        "DRIVERS": "Drivers Editor",
        "CLIP_EDITOR": "Movie Clip Editor",
        "DOPESHEET": "Dope Sheet",
        "NLA_EDITOR": "Nonlinear Animation",
        "ASSETS": "Asset Browser",
        "SPREADSHEET": "Spreadsheet Editor",
        "TEXT_EDITOR": "Text Editor",
        "CONSOLE": "Console",
        "INFO": "Info",
        "TOPBAR": "Topbar",
        "STATUSBAR": "Statusbar",
        "OUTLINER": "Outliner",
        "PROPERTIES": "Properties",
        "PREFERENCES": "Preferences",
        "HEADER": "Header",
        "FOOTER": "Footer",
        "TOOL_PROPS": "Tool Properties",
    }

    return type_name.get(area.ui_type, "")


def draw_paragraph(
    layout: bpy.types.UILayout,
    text,
    alignment="LEFT",
    line_spacing=0.9,
    row_length=40,
    margin=2,
):
    col = layout.column()
    col.scale_y = line_spacing
    for l, _ in iter_lines(text):
        for r in line_to_rows(l, row_length):
            if not r:
                col.separator(factor=1)
                continue

            row = col.row()
            row.alignment = alignment
            row.label(text=r)

    if margin > 0:
        col.separator(factor=margin)


def draw_icon(
    layout: bpy.types.UILayout, icon, scale=2, spacing_top=1, spacing_bottom=1
):
    if spacing_top > 0:
        layout.separator(factor=spacing_top)

    layout.template_icon(icon_value=get_icon_value(icon), scale=scale)

    if spacing_bottom > 0:
        layout.separator(factor=spacing_bottom)


def draw_text_input(layout: bpy.types.UILayout, data, prop, row_length=55, spacing=0.5):
    text = getattr(data, prop)
    col = layout.column(align=True)
    if len(text) > row_length:
        draw_exceeding_text_box(col, spacing, text, row_length)
    col.prop(data, prop, text="")


def draw_exceeding_text_box(col, spacing, text, row_length):
    box = col.box()
    row = box.row(align=True)
    row.separator(factor=spacing)
    text_col = row.column()
    text_col.scale_y = 0.9
    draw_text(text_col, text, row_length)
    text_col.separator(factor=0.1)


def close_dialog(context, event):
    try:
        __import__("pynput")
    except ImportError:
        x, y = event.mouse_x, event.mouse_y
        context.window.cursor_warp(10, 10)
        move_back = lambda: context.window.cursor_warp(x, y)
        bpy.app.timers.register(move_back, first_interval=0.0001)
        return

    from pynput.keyboard import Key, Controller

    keyboard = Controller()
    keyboard.press(Key.left)
    keyboard.release(Key.left)


def temp_mouse_move(context, event, position):
    mouse_co = (event.mouse_x, event.mouse_y)
    context.window.cursor_warp(position[0], position[1])
    move_back = lambda: context.window.cursor_warp(mouse_co[0], mouse_co[1])
    bpy.app.timers.register(move_back, first_interval=0.0001)


def text_box(
    layout: bpy.types.UILayout, rows, data, prop, line_spacing, label=None, icon=None
):
    box = layout.box()
    row = box.row(align=True)
    expand = getattr(data, prop)
    row.prop(
        data,
        prop,
        text="",
        icon="DOWNARROW_HLT" if expand else "RIGHTARROW",
        emboss=False,
    )

    if icon is not None:
        row.label(icon=icon)
    if label is not None:
        row.separator(factor=0.5)
        row.label(text=label)

    if expand:
        col = box.column()
        col.separator(factor=1)
        col.scale_y = line_spacing

        for r in rows:
            col.label(text=r)

    return row


def not_found(layout: bpy.types.UILayout, text):
    row = layout.row(align=True)
    row.label(icon="GHOST_DISABLED")
    row.separator(factor=0.5)
    row.label(text=text)


def heading_layout(self, layout, text):
    row = layout.row(align=True)
    col = row.column()
    col.enabled = False
    col.label(text=text)
    layout.separator(factor=1)


def settings_layout(layout, factor=0.5, align=False, offset=0, indentation=0):
    if indentation > 0:
        offset = indentation * 2.4

    split = layout.split(factor=factor, align=align)
    if offset > 0:
        row: bpy.types.UILayout = split.row(align=align)
        row.separator(factor=offset)
        l_col = row.column(align=align)
    else:
        l_col: bpy.types.UILayout = split.column(align=align)
    r_col: bpy.types.UILayout = split.column(align=align)
    return l_col, r_col


def aligned_box(layout: bpy.types.UILayout):
    box = layout.box()
    box.scale_y = 0.55
    box.scale_x = 0.85
    return box


def progress_bar(context, layout: bpy.types.UILayout, factor):
    if bpy.app.version < (4, 0, 0):
        wm = context.window_manager
        wm.blendai.progress_bar = int(min(99, factor * 100))
        layout.prop(wm.blendai, "progress_bar", text="")
    else:
        layout.progress(factor=factor, text=f"{min(99, factor * 100):.0f}%")


def draw_discord_invite(layout, text="Join our Discord", show_icon=True):
    layout.operator(
        "wm.url_open", text=text, icon_value=icon_id("discord.png") if show_icon else 0
    ).url = "https://discord.gg/TYFq6VKXCN"


def draw_send_email(
    layout: bpy.types.UILayout,
    text="Send an Email",
    subject="",
    body="",
    include_addon_info=True,
    show_icon=True,
):
    op = layout.operator(
        f"{addon_id}.email",
        text=text,
        icon_value=icon_id("email.png") if show_icon else 0,
    )
    op.include_addon_info = include_addon_info
    op.subject = subject
    op.body = body
